@@ -3,12 +3,16 @@ The top-level augur command which dispatches to subcommands.
 """
 
 import argparse
-import re
 import os
 import sys
 import importlib
+import traceback
+from textwrap import dedent
 from types import SimpleNamespace
-from .utils import first_line
+
+from .errors import AugurError
+from .io import print_err
+from .argparse_ import add_command_subparsers, add_default_command
 
 recursion_limit = os.environ.get("AUGUR_RECURSION_LIMIT")
 if recursion_limit:
@@ -35,7 +39,8 @@ command_strings = [
     "export",
     "validate",
     "version",
-    "import"
+    "import_",
+    "measurements",
 ]
 
 COMMANDS = [importlib.import_module('augur.' + c) for c in command_strings]
@@ -45,26 +50,11 @@ def make_parser():
         prog        = "augur",
         description = "Augur: A bioinformatics toolkit for phylogenetic analysis.")
 
-    subparsers = parser.add_subparsers()
-
     add_default_command(parser)
     add_version_alias(parser)
 
-    for command in COMMANDS:
-        # Add a subparser for each command.
-        subparser = subparsers.add_parser(
-            command_name(command),
-            help        = first_line(command.__doc__),
-            description = command.__doc__)
-
-        subparser.set_defaults(__command__ = command)
-
-        # Let the command register arguments on its subparser.
-        command.register_arguments(subparser)
-
-        # Use the same formatting class for every command for consistency.
-        # Set here to avoid repeating it in every command's register_parser().
-        subparser.formatter_class = argparse.ArgumentDefaultsHelpFormatter
+    subparsers = parser.add_subparsers()
+    add_command_subparsers(subparsers, COMMANDS)
 
     return parser
 
@@ -73,21 +63,24 @@ def run(argv):
     args = make_parser().parse_args(argv)
     try:
         return args.__command__.run(args)
+    except AugurError as e:
+        print_err(f"ERROR: {e}")
+        sys.exit(2)
     except RecursionError:
-        print("FATAL: Maximum recursion depth reached. You can set the env variable AUGUR_RECURSION_LIMIT to adjust this (current limit: {})".format(sys.getrecursionlimit()))
+        print_err("FATAL: Maximum recursion depth reached. You can set the env variable AUGUR_RECURSION_LIMIT to adjust this (current limit: {})".format(sys.getrecursionlimit()))
+        sys.exit(2)
+    except FileNotFoundError as e:
+        print_err(f"ERROR: {e.strerror}: '{e.filename}'")
+        sys.exit(2)
+    except Exception:
+        traceback.print_exc(file=sys.stderr)
+        print_err("\n")
+        print_err(dedent("""\
+            An error occurred (see above) that has not been properly handled by Augur.
+            To report this, please open a new issue including the original command and the error above:
+                <https://github.com/nextstrain/augur/issues/new/choose>
+            """))
         sys.exit(2)
-
-
-def add_default_command(parser):
-    """
-    Sets the default command to run when none is provided.
-    """
-    class default_command():
-        def run(args):
-            parser.print_help()
-            return 2
-
-    parser.set_defaults(__command__ = default_command)
 
 
 def add_version_alias(parser):
@@ -108,17 +101,3 @@ def add_version_alias(parser):
         nargs  = 0,
         help   = argparse.SUPPRESS,
         action = run_version_command)
-
-
-def command_name(command):
-    """
-    Returns a short name for a command module.
-    """
-
-    def remove_prefix(prefix, string):
-        return re.sub('^' + re.escape(prefix), '', string)
-
-    package     = command.__package__
-    module_name = command.__name__
-
-    return remove_prefix(package, module_name).lstrip(".").replace("_", "-")
