@@ -1,6 +1,5 @@
 """
-Augur Version 1-specific helpers for exporting JSON files suitable for
-visualization with auspice.
+Export version 1 JSON schema (separate meta and tree JSONs) for visualization with Auspice
 """
 
 import os, sys
@@ -10,7 +9,8 @@ import numpy as np
 from Bio import Phylo
 from argparse import SUPPRESS
 from collections import defaultdict
-from .utils import read_metadata, read_node_data, write_json, read_config, read_lat_longs, read_colors
+from .io import read_metadata
+from .utils import read_node_data, write_json, read_config, read_lat_longs, read_colors
 
 def convert_tree_to_json_structure(node, metadata, div=0, strains=None):
     """
@@ -227,7 +227,7 @@ def construct_author_info_v1(metadata, tree, nodes):
 
     author_info = defaultdict(lambda: {"n": 0})
     no_authors = 0
-    for strain, data in metadata.items():
+    for strain, data in metadata.iterrows():
         if "authors" not in data:
             no_authors += 1
             continue
@@ -260,17 +260,17 @@ def add_tsv_metadata_to_nodes(nodes, meta_tsv, meta_json, extra_fields=['authors
         fields = [x for x in meta_json["color_options"].keys() if x != "gt"] + extra_fields
     else:
         fields = list(extra_fields)
-        
+
     if "geo" in meta_json:
         fields += meta_json["geo"]
 
     for strain, node in nodes.items():
-        if strain not in meta_tsv:
+        if strain not in meta_tsv.index:
             continue
         for field in fields:
             # Allow fields to have value of 0! - but prevent from having value of "" (breaks auspice v1)
-            if field not in node and field in meta_tsv[strain] and (meta_tsv[strain][field] or meta_tsv[strain][field]==0):
-                node[field] = meta_tsv[strain][field]
+            if field not in node and field in meta_tsv.columns and (meta_tsv.at[strain, field] or meta_tsv.at[strain, field]==0):
+                node[field] = meta_tsv.at[strain, field]
 
 
 def get_root_sequence(root_node, ref=None, translations=None):
@@ -330,15 +330,15 @@ def add_option_args(parser):
     return options
 
 
-def register_arguments_v1(subparsers):
-    # V1 sub-command
-    v1 = subparsers.add_parser('v1', help="Export version 1 JSON schema (separate meta and tree JSONs)")
-    v1_core = add_core_args(v1)
-    v1_options = add_option_args(v1)
-    v1.add_argument("--v1", help=SUPPRESS, default=True)
+def register_parser(parent_subparsers):
+    parser = parent_subparsers.add_parser("v1", help=__doc__)
+    add_core_args(parser)
+    add_option_args(parser)
+    parser.add_argument("--v1", help=SUPPRESS, default=True)
+    return parser
 
 
-def run_v1(args):
+def run(args):
     T = Phylo.read(args.tree, 'newick')
     node_data = read_node_data(args.node_data) # args.node_data is an array of multiple files (or a single file)
     nodes = node_data["nodes"] # this is the per-node metadata produced by various augur modules
@@ -363,7 +363,7 @@ def run_v1(args):
 
     meta_json = read_config(args.auspice_config)
     ensure_config_is_v1(meta_json)
-    meta_tsv, _ = read_metadata(args.metadata)
+    meta_tsv = read_metadata(args.metadata)
     add_tsv_metadata_to_nodes(nodes, meta_tsv, meta_json)
 
     tree_layout(T)
