@@ -1,15 +1,17 @@
 """
-Export JSON files suitable for visualization with auspice.
+Export version 2 JSON schema for visualization with Auspice
 """
 from pathlib import Path
 import os, sys
 import time
-from collections import defaultdict, deque
+from collections import defaultdict, deque, OrderedDict
 import warnings
 import numbers
 import re
 from Bio import Phylo
-from .utils import read_metadata, read_node_data, write_json, read_config, read_lat_longs, read_colors
+
+from .io import read_metadata
+from .utils import read_node_data, write_json, read_config, read_lat_longs, read_colors
 from .validate import export_v2 as validate_v2, auspice_config_v2 as validate_auspice_config_v2, ValidateError
 
 # Set up warnings & exceptions
@@ -45,6 +47,67 @@ def configure_warnings():
 class InvalidOption(Exception):
     pass
 
+class CustomOrderedDict(OrderedDict):
+    """
+    Similar to OrderedDict but will convert dictionaries (and dictionaries of dictionaries)
+    into (nested) CustomOrderedDicts.
+    Encountered lists of dicts will be converted to lists of CustomOrderedDict but we will not
+    recursively explore nested lists.
+    Tuples and other iterators are not explored.
+    """
+    def __init__(self, *args):
+        super().__init__(*args)
+        for key in self:
+            if isinstance(self[key], dict) and not isinstance(self[key], OrderedDict):
+                self[key] = CustomOrderedDict(self[key])
+            elif isinstance(self[key], list):
+                self[key] = [
+                    (CustomOrderedDict(el) if (isinstance(el, dict) and not isinstance(el, OrderedDict)) else el)
+                    for el in self[key]
+                ]
+    def set_order(self, *order):
+        """
+        changes the order of keys to match those specified in `order` as much
+        as possible. Missing keys are ignored. Extra keys will come after those
+        specified in `order`.
+        """
+        for key in reversed(order):
+            self.move_to_end_if_present(key, last=False)
+    def move_to_end_if_present(self, key, **kwargs):
+        try:
+            self.move_to_end(key, **kwargs)
+        except KeyError:
+            pass
+
+
+def orderKeys(data):
+    """
+    converts the data dict (where keys are inherently unordered) into an
+    OrderedDict where keys are nicely ordered for human eyes to scan the
+    data when written to JSON. The ordering (mostly) mirrors the schema.
+    """
+    od = CustomOrderedDict(data)
+    od.set_order("version", "meta", "tree")
+    if "meta" in od:
+        od["meta"].set_order("title", "updated", "build_url", "data_provenance", "maintainers")
+        for coloring in od['meta'].get('colorings', []):
+            coloring.set_order("key", "title", "type", "scale", "legend")
+    def order_nodes(node):
+        """recursive function to order nodes in a (sub)tree"""
+        node.set_order("name", "node_attrs", "branch_attrs")
+        # children often a _large_ object and it improves readability if this comes last in the node
+        node.move_to_end_if_present("children")
+        if "node_attrs" in node:
+            node["node_attrs"].set_order("div", "num_date")
+        for child in node.get("children", []):
+            order_nodes(child)
+    if isinstance(od.get("tree"), list):
+        for subtree in od['tree']:
+            order_nodes(subtree)
+    elif isinstance(od.get("tree"), dict):
+        order_nodes(od['tree'])
+    return od
+
 def convert_tree_to_json_structure(node, metadata, div=0):
     """
     converts the Biopython tree structure to a dictionary that can
@@ -173,7 +236,7 @@ def set_colorings(data_json, config, command_line_colorings, metadata_names, nod
 
         if config.get(key, {}).get("type"):
             t = config.get(key).get("type")
-            allowedTypes = ["continuous", "ordinal", "categorical", "boolean"]
+            allowedTypes = ["continuous", "temporal", "ordinal", "categorical", "boolean"]
             if t not in allowedTypes:
                 warn("[config file] In trait {}, coloring type '{}' is not valid. Please choose from: '{}'. This trait has been excluded!".format(key, t, ", ".join(allowedTypes)))
                 raise InvalidOption()
@@ -504,6 +567,7 @@ def set_data_provenance(data_json, config):
     config : dict
         config JSON with an expected ``data_provenance`` key
 
+
     >>> config = {"data_provenance": [{"name": "GISAID"}, {"name": "INSDC"}]}
     >>> data_json = {"meta": {}}
     >>> set_data_provenance(data_json, config)
@@ -754,51 +818,18 @@ def node_data_prop_is_normal_trait(name):
 
     return True
 
-def get_root_sequence(root_node, ref=None, translations=None):
-    '''
-    create a json structure that contains the sequence of the root, both as
-    nucleotide and as translations. This allows look-up of the sequence for
-    all states, including those that are not variable.
 
-    Parameters
-    ----------
-    root_node : dict
-    	data associated with the node
-    ref : str, optional
-        filename of the root sequence
-    translations : str, optional
-        file name of translations
+def register_parser(parent_subparsers):
+    parser = parent_subparsers.add_parser("v2", help=__doc__)
 
-    Returns
-    -------
-    dict
-        dict of nucleotide sequence and translations
-    '''
-    root_sequence = {}
-    if ref and translations:
-        from Bio import SeqIO
-        refseq = SeqIO.read(ref, 'fasta')
-        root_sequence['nuc']=str(refseq.seq)
-        for gene in SeqIO.parse(translations, 'fasta'):
-            root_sequence[gene.id] = str(gene.seq)
-    else:
-        root_sequence["nuc"] = root_node["sequence"]
-        root_sequence.update(root_node["aa_sequences"])
-
-    return root_sequence
-
-
-def register_arguments_v2(subparsers):
-    v2 = subparsers.add_parser("v2", help="Export version 2 JSON schema")
-
-    required = v2.add_argument_group(
+    required = parser.add_argument_group(
         title="REQUIRED"
     )
     required.add_argument('--tree','-t', metavar="newick", required=True, help="Phylogenetic tree, usually output from `augur refine`")
     required.add_argument('--node-data', metavar="JSON", required=True, nargs='+', help="JSON files containing metadata for nodes in the tree")
     required.add_argument('--output', metavar="JSON", required=True, help="Ouput file (typically for visualisation in auspice)")
 
-    config = v2.add_argument_group(
+    config = parser.add_argument_group(
         title="DISPLAY CONFIGURATION",
         description="These control the display settings for auspice. \
             You can supply a config JSON (which has all available options) or command line arguments (which are more limited but great to get started). \
@@ -811,22 +842,23 @@ def register_arguments_v2(subparsers):
     config.add_argument('--description', metavar="description.md", help="Markdown file with description of build and/or acknowledgements to be displayed by Auspice")
     config.add_argument('--geo-resolutions', metavar="trait", nargs='+', help="Geographic traits to be displayed on map")
     config.add_argument('--color-by-metadata', metavar="trait", nargs='+', help="Metadata columns to include as coloring options")
-    config.add_argument('--panels', metavar="panels", nargs='+', choices=['tree', 'map', 'entropy', 'frequencies'], help="Restrict panel display in auspice. Options are %(choices)s. Ignore this option to display all available panels.")
+    config.add_argument('--panels', metavar="panels", nargs='+', choices=['tree', 'map', 'entropy', 'frequencies', 'measurements'], help="Restrict panel display in auspice. Options are %(choices)s. Ignore this option to display all available panels.")
 
-    optional_inputs = v2.add_argument_group(
+    optional_inputs = parser.add_argument_group(
         title="OPTIONAL INPUT FILES"
     )
     optional_inputs.add_argument('--metadata', metavar="FILE", help="Additional metadata for strains in the tree, as CSV or TSV")
     optional_inputs.add_argument('--colors', metavar="FILE", help="Custom color definitions, one per line in the format `TRAIT_TYPE\\tTRAIT_VALUE\\tHEX_CODE`")
     optional_inputs.add_argument('--lat-longs', metavar="TSV", help="Latitudes and longitudes for geography traits (overrides built in mappings)")
 
-    optional_settings = v2.add_argument_group(
+    optional_settings = parser.add_argument_group(
         title="OPTIONAL SETTINGS"
     )
     optional_settings.add_argument('--minify-json', action="store_true", help="export JSONs without indentation or line returns")
     optional_settings.add_argument('--include-root-sequence', action="store_true", help="Export an additional JSON containing the root sequence (reference sequence for vcf) used to identify mutations. The filename will follow the pattern of <OUTPUT>_root-sequence.json for a main auspice JSON of <OUTPUT>.json")
+    optional_settings.add_argument('--skip-validation', action="store_true", help="skip validation of input/output files. Use at your own risk!")
 
-    return v2
+    return parser
 
 
 def set_display_defaults(data_json, config):
@@ -936,12 +968,13 @@ def get_config(args):
     if not args.auspice_config:
         return {}
     config = read_config(args.auspice_config)
-    try:
-        print("Validating config file {} against the JSON schema".format(args.auspice_config))
-        validate_auspice_config_v2(args.auspice_config)
-    except ValidateError:
-        print("Validation of {} failed. Please check the formatting of this file & refer to the augur documentation for further help. ".format(args.auspice_config))
-        sys.exit(2)
+    if not args.skip_validation:
+        try:
+            print("Validating config file {} against the JSON schema".format(args.auspice_config))
+            validate_auspice_config_v2(args.auspice_config)
+        except ValidateError:
+            print("Validation of {} failed. Please check the formatting of this file & refer to the augur documentation for further help. ".format(args.auspice_config))
+            sys.exit(2)
     # Print a warning about the inclusion of "vaccine_choices" which are _unused_ by `export v2`
     # (They are in the schema as this allows v1-compat configs to be used)
     if config.get("vaccine_choices"):
@@ -949,23 +982,29 @@ def get_config(args):
         del config["vaccine_choices"]
     return config
 
-def run_v2(args):
+def run(args):
     configure_warnings()
     data_json = {"version": "v2", "meta": {"updated": time.strftime('%Y-%m-%d')}}
 
     #load input files
     try:
-        node_data_file = read_node_data(args.node_data) # node_data_files is an array of multiple files (or a single file)
+        node_data_file = read_node_data(args.node_data, skip_validation=args.skip_validation) # node_data_files is an array of multiple files (or a single file)
     except FileNotFoundError:
         print(f"ERROR: node data file ({args.node_data}) does not exist")
         sys.exit(2)
 
     if args.metadata is not None:
         try:
-            metadata_file, _ = read_metadata(args.metadata)
+            metadata_file = read_metadata(args.metadata).to_dict(orient="index")
+            for strain in metadata_file.keys():
+                if "strain" not in metadata_file[strain]:
+                    metadata_file[strain]["strain"] = strain
         except FileNotFoundError:
-            print(f"ERROR: meta data file ({args.metadata}) does not exist")
+            print(f"ERROR: meta data file ({args.metadata}) does not exist", file=sys.stderr)
             sys.exit(2)
+        except Exception as error:
+            print(f"ERROR: {error}", file=sys.stderr)
+            sys.exit(1)
     else:
         metadata_file = {}
 
@@ -1005,9 +1044,13 @@ def run_v2(args):
     set_panels(data_json, config, args.panels)
     set_data_provenance(data_json, config)
 
+    # pass through any extensions block in the auspice config JSON without any changes / checking
+    if config.get("extensions"):
+        data_json["meta"]["extensions"] = config["extensions"]
+
     # Write outputs - the (unified) dataset JSON intended for auspice & perhaps the ref root-sequence JSON
     indent = {"indent": None} if args.minify_json else {}
-    write_json(data=data_json, file_name=args.output, include_version=False, **indent)
+    write_json(data=orderKeys(data_json), file_name=args.output, include_version=False, **indent)
 
     if args.include_root_sequence:
         if 'reference' in node_data:
@@ -1022,7 +1065,8 @@ def run_v2(args):
             fatal("Root sequence output was requested, but the node data provided is missing a 'reference' key.")
 
     # validate outputs
-    validate_data_json(args.output)
+    if not args.skip_validation:
+        validate_data_json(args.output)
 
     if deprecationWarningsEmitted:
         print("\n------------------------")
