@@ -2,19 +2,20 @@
 infer frequencies of mutations or clades
 """
 import json, os, sys
-import datetime
 import numpy as np
 from collections import defaultdict
 from Bio import Phylo, AlignIO
 from Bio.Align import MultipleSeqAlignment
-import treetime.utils
 
 from .frequency_estimators import get_pivots, alignment_frequencies, tree_frequencies
 from .frequency_estimators import AlignmentKdeFrequencies, TreeKdeFrequencies, TreeKdeFrequenciesError
-from .utils import read_metadata, read_node_data, write_json, get_numerical_dates
+from .dates import numeric_date_type, SUPPORTED_DATE_HELP_TEXT, get_numerical_dates
+from .io import read_metadata
+from .utils import read_node_data, write_json
 
 
-def register_arguments(parser):
+def register_parser(parent_subparsers):
+    parser = parent_subparsers.add_parser("frequencies", help=__doc__)
     # Shared arguments
     parser.add_argument('--method', choices=["diffusion", "kde"], required=True,
                         help="method by which frequencies should be estimated")
@@ -26,10 +27,10 @@ def register_arguments(parser):
                         help="number of units between pivots")
     parser.add_argument("--pivot-interval-units", type=str, default="months", choices=['months', 'weeks'],
                         help="space pivots by months (default) or by weeks")
-    parser.add_argument('--min-date', type=numeric_date,
-                        help="date to begin frequencies calculations; may be specified as an Augur-style numeric date (with the year as the integer part) or YYYY-MM-DD")
-    parser.add_argument('--max-date', type=numeric_date,
-                        help="date to end frequencies calculations; may be specified as an Augur-style numeric date (with the year as the integer part) or YYYY-MM-DD")
+    parser.add_argument('--min-date', type=numeric_date_type,
+                        help=f"date to begin frequencies calculations; may be specified as: {SUPPORTED_DATE_HELP_TEXT}")
+    parser.add_argument('--max-date', type=numeric_date_type,
+                        help=f"date to end frequencies calculations; may be specified as: {SUPPORTED_DATE_HELP_TEXT}")
 
     # Tree-specific arguments
     parser.add_argument('--tree', '-t', type=str,
@@ -71,6 +72,7 @@ def register_arguments(parser):
                         help="format to export frequencies JSON depending on the viewing interface")
     parser.add_argument('--output', '-o', type=str,
                         help='JSON file to save estimated frequencies to')
+    return parser
 
 
 def format_frequencies(freq):
@@ -78,7 +80,7 @@ def format_frequencies(freq):
 
 
 def run(args):
-    metadata, columns = read_metadata(args.metadata)
+    metadata = read_metadata(args.metadata)
     dates = get_numerical_dates(metadata, fmt='%Y-%m-%d')
     stiffness = args.stiffness
     inertia = args.inertia
@@ -103,7 +105,7 @@ def run(args):
 
             # Annotate tips with metadata to enable filtering and weighting of
             # frequencies by metadata attributes.
-            for key, value in metadata[tip.name].items():
+            for key, value in metadata.loc[tip.name].items():
                 tip.attr[key] = value
 
         if args.method == "diffusion":
@@ -230,19 +232,3 @@ def run(args):
 
         write_json(frequencies, args.output)
         print("mutation frequencies written to", args.output, file=sys.stdout)
-
-
-def numeric_date(date):
-    """
-    Converts the given *date* string to a :py:class:`float`.
-    *date* may be given as a number (a float) with year as the integer part, or
-    in the YYYY-MM-DD (ISO 8601) syntax.
-    >>> numeric_date("2020.42")
-    2020.42
-    >>> numeric_date("2020-06-04")
-    2020.42486...
-    """
-    try:
-        return float(date)
-    except ValueError:
-        return treetime.utils.numeric_date(datetime.date(*map(int, date.split("-", 2))))
