@@ -2,10 +2,13 @@
 Parse delimited fields from FASTA sequence names into a TSV and FASTA file.
 """
 import pandas as pd
+import sys
 
 from .io import open_file, read_sequences, write_sequences
+from .dates import get_numerical_date_from_value
+from .errors import AugurError
 
-forbidden_chactacters = str.maketrans(
+forbidden_characters = str.maketrans(
     {' ': None,
      '(': '_',
      ')': '_',
@@ -38,8 +41,17 @@ def fix_dates(d, dayfirst=True):
             return "%d-%02d-XX"%(dto.year, dto.month)
         else:
             return "%d-%02d-%02d"%(dto.year, dto.month, dto.day)
-    except Exception as e:
-        print("WARNING: unable to parse %s as date"%d, e)
+    except ValueError as e:
+        # If the date can't be parsed by pandas above or as our own ambiguous
+        # date format (e.g., "2020-XX-XX"), let the user know.
+        try:
+            parsed_date = get_numerical_date_from_value(d, "%Y-%m-%d")
+        except ValueError:
+            parsed_date = None
+
+        if parsed_date is None:
+            print("WARNING: unable to parse %s as date"%d, e, file=sys.stderr)
+
         return d
 
 def prettify(x, trim=0, camelCase=False, etal=None, removeComma=False):
@@ -68,7 +80,7 @@ def prettify(x, trim=0, camelCase=False, etal=None, removeComma=False):
     return res
 
 
-def parse_sequence(sequence, fields, strain_key="strain", separator="|", prettify_fields=None, fix_dates=None):
+def parse_sequence(sequence, fields, strain_key="strain", separator="|", prettify_fields=None, fix_dates_format=None):
     """Parse a single sequence record into a sequence record and associated metadata.
 
     Parameters
@@ -88,7 +100,7 @@ def parse_sequence(sequence, fields, strain_key="strain", separator="|", prettif
     prettify_fields : list or tuple
         a list of field names for which the values in those fields should be prettified.
 
-    fix_dates : str
+    fix_dates_format : str
         parse "date" field into the requested canonical format ("dayfirst" or "monthfirst").
 
     Returns
@@ -103,7 +115,7 @@ def parse_sequence(sequence, fields, strain_key="strain", separator="|", prettif
     sequence_fields = map(str.strip, sequence.description.split(separator))
     metadata = dict(zip(fields, sequence_fields))
 
-    tmp_name = metadata[strain_key].translate(forbidden_chactacters)
+    tmp_name = metadata[strain_key].translate(forbidden_characters)
     sequence.name = sequence.id = tmp_name
     sequence.description = ''
 
@@ -117,7 +129,7 @@ def parse_sequence(sequence, fields, strain_key="strain", separator="|", prettif
     if fix_dates and 'date' in metadata:
         metadata['date'] = fix_dates(
             metadata['date'],
-            dayfirst=fix_dates=='dayfirst'
+            dayfirst=fix_dates_format=='dayfirst'
         )
 
     metadata["strain"] = sequence.id
@@ -125,7 +137,8 @@ def parse_sequence(sequence, fields, strain_key="strain", separator="|", prettif
     return sequence, metadata
 
 
-def register_arguments(parser):
+def register_parser(parent_subparsers):
+    parser = parent_subparsers.add_parser("parse", help=__doc__)
     parser.add_argument('--sequences', '-s', required=True, help="sequences in fasta or VCF format")
     parser.add_argument('--output-sequences', help="output sequences file")
     parser.add_argument('--output-metadata', help="output metadata file")
@@ -134,6 +147,7 @@ def register_arguments(parser):
     parser.add_argument('--separator', default='|', help="separator of fasta header")
     parser.add_argument('--fix-dates', choices=['dayfirst', 'monthfirst'],
                                 help="attempt to parse non-standard dates and output them in standard YYYY-MM-DD format")
+    return parser
 
 
 def run(args):
@@ -165,6 +179,8 @@ def run(args):
                 args.prettify_fields,
                 args.fix_dates
             )
+            if sequence_record.id in meta_data:
+                raise AugurError(f"Duplicate found for '{sequence_record.id}'.")
             meta_data[sequence_record.id] = sequence_metadata
 
             sequences_written = write_sequences(
