@@ -6,7 +6,8 @@ import numpy as np
 from collections import defaultdict
 import os, sys
 import pandas as pd
-from .utils import read_metadata, write_json, get_json_name
+from .io import read_metadata
+from .utils import write_json, get_json_name
 TINY = 1e-12
 
 def mugration_inference(tree=None, seq_meta=None, field='country', confidence=True,
@@ -18,7 +19,7 @@ def mugration_inference(tree=None, seq_meta=None, field='country', confidence=Tr
     ----------
     tree : str
         name of tree file
-    seq_meta : dict
+    seq_meta : pandas.DataFrame
         meta data associated with sequences
     field : str, optional
         meta data field to use
@@ -49,18 +50,18 @@ def mugration_inference(tree=None, seq_meta=None, field='country', confidence=Tr
     T = Phylo.read(tree, 'newick')
     traits = {}
     nodes = {n.name:n for n in T.get_terminals()}
-    for name, meta in seq_meta.items():
+    for name, meta in seq_meta.iterrows():
         if field in meta and name in nodes:
             traits[name] = meta[field]
     unique_states = list(set(traits.values()))
 
     if len(unique_states)==0:
-        print("WARNING: no states found for discrete state reconstruction.")
+        print("WARNING: no states found for discrete state reconstruction.", file=sys.stderr)
         for node in T.find_clades():
             node.__setattr__(field, None)
         return T, None, {}
     elif len(unique_states)==1:
-        print("WARNING: only one state found for discrete state reconstruction:", unique_states)
+        print("WARNING: only one state found for discrete state reconstruction:", unique_states, file=sys.stderr)
         for node in T.find_clades():
             node.__setattr__(field, unique_states[0])
         return T, None, {}
@@ -69,11 +70,11 @@ def mugration_inference(tree=None, seq_meta=None, field='country', confidence=Tr
             reconstruct_discrete_traits(T, traits, missing_data=missing,
                  sampling_bias_correction=sampling_bias_correction, weights=weights)
     else:
-        print("ERROR: 300 or more distinct discrete states found. TreeTime is currently not set up to handle that many states.")
+        print("ERROR: 300 or more distinct discrete states found. TreeTime is currently not set up to handle that many states.", file=sys.stderr)
         sys.exit(1)
 
     if tt is None:
-        print("ERROR in discrete state reconstruction in TreeTime. Please look for errors above.")
+        print("ERROR in discrete state reconstruction in TreeTime. Please look for errors above.", file=sys.stderr)
         sys.exit(1)
 
     # attach inferred states as e.g. node.region = 'africa'
@@ -96,7 +97,7 @@ def mugration_inference(tree=None, seq_meta=None, field='country', confidence=Tr
     return tt.tree, tt.gtr, letter_to_state
 
 
-def register_arguments(parser):
+def register_parser(parent_subparsers):
     """Add subcommand specific arguments
 
     Parameters
@@ -104,6 +105,7 @@ def register_arguments(parser):
     parser : argparse
         subcommand argument parser
     """
+    parser = parent_subparsers.add_parser("traits", help=__doc__)
     parser.add_argument('--tree', '-t', required=True, help="tree to perform trait reconstruction on")
     parser.add_argument('--metadata', required=True, metavar="FILE", help="table with metadata, as CSV or TSV")
     parser.add_argument('--weights', required=False, help="tsv/csv table with equilibrium probabilities of discrete states")
@@ -119,6 +121,8 @@ def register_arguments(parser):
                              '(or rather the time spent in a particular state on the tree)')
     parser.add_argument('--output-node-data', type=str, help='name of JSON file to save trait inferences to')
     parser.epilog = "Note that missing data must be represented by a `?` character. Missing data will currently be inferred."
+    return parser
+
 
 def run(args):
     """run mugration inference
@@ -129,18 +133,18 @@ def run(args):
         command line arguments are parsed by argparse
     """
     tree_fname = args.tree
-    traits, columns = read_metadata(args.metadata)
+    traits = read_metadata(args.metadata)
 
     from Bio import Phylo
     T = Phylo.read(tree_fname, 'newick')
     missing_internal_node_names = [n.name is None for n in T.get_nonterminals()]
     if np.all(missing_internal_node_names):
-        print("\n*** WARNING: Tree has no internal node names!")
-        print("*** Without internal node names, ancestral traits can't be linked up to the correct node later.")
-        print("*** If you want to use 'augur export' later, re-run this command with the output of 'augur refine'.")
-        print("*** If you haven't run 'augur refine', you can add node names to your tree by running:")
-        print("*** augur refine --tree %s --output-tree <filename>.nwk"%(tree_fname) )
-        print("*** And use <filename>.nwk as the tree when running 'ancestral', 'translate', and 'traits'")
+        print("\n*** WARNING: Tree has no internal node names!", file=sys.stderr)
+        print("*** Without internal node names, ancestral traits can't be linked up to the correct node later.", file=sys.stderr)
+        print("*** If you want to use 'augur export' later, re-run this command with the output of 'augur refine'.", file=sys.stderr)
+        print("*** If you haven't run 'augur refine', you can add node names to your tree by running:", file=sys.stderr)
+        print("*** augur refine --tree %s --output-tree <filename>.nwk" % (tree_fname), file=sys.stderr)
+        print("*** And use <filename>.nwk as the tree when running 'ancestral', 'translate', and 'traits'", file=sys.stderr)
 
     if args.weights:
         weight_dict = {c:{} for c in args.columns}
@@ -174,10 +178,16 @@ def run(args):
             continue
 
         for node in T.find_clades():
-            mugration_states[node.name][column] = node.__getattribute__(column)
+            mugration_states[node.name][column] = getattr(node, column)
+
             if args.confidence:
-                mugration_states[node.name][column+'_confidence'] = node.__getattribute__(column+'_confidence')
-                mugration_states[node.name][column+'_entropy'] = node.__getattribute__(column+'_entropy')
+                confidence = getattr(node, f"{column}_confidence", None)
+                if confidence is not None:
+                    mugration_states[node.name][f"{column}_confidence"] = confidence
+
+                entropy = getattr(node, f"{column}_entropy", None)
+                if entropy is not None:
+                    mugration_states[node.name][f"{column}_entropy"] = entropy
 
         if gtr:
             # add gtr models to json structure for export
