@@ -15,8 +15,8 @@ import numpy as np
 from treetime.vcf_utils import read_vcf
 from pathlib import Path
 
-from .io import read_sequences
-from .utils import run_shell_command, nthreads_value, shquote, load_mask_sites
+from .io import read_sequences, run_shell_command, shquote
+from .utils import nthreads_value, load_mask_sites
 
 DEFAULT_ARGS = {
     "fasttree": "-nt -nosupport",
@@ -59,6 +59,7 @@ def check_conflicting_args(tree_builder_args, defaults):
     ConflictingArgumentsException
         When any user-provided arguments match those in the defaults.
 
+
     >>> defaults = ("-nt", "-m", "-s")
     >>> check_conflicting_args("-czb -n 2", defaults)
     >>> check_conflicting_args("-czb -nt 2", defaults)
@@ -329,6 +330,10 @@ def write_out_informative_fasta(compress_seq, alignment, stripFile=None):
 
     #Rotate and convert to SeqRecord
     sites = np.asarray(sites)
+    if len(sites.shape)!=2:
+        print("ERROR: NO VALID SITES REMAIN AFTER IGNORING UNCALLED SITES")
+        raise Exception
+
     align = np.rot90(sites)
     seqNamesCorr = list(reversed(seqNames))
     toFasta = [ SeqRecord(id=seqNamesCorr[i], seq=Seq("".join(align[i])), description='') for i in range(len(sequences.keys()))]
@@ -364,7 +369,7 @@ def mask_sites_in_multiple_sequence_alignment(alignment_file, excluded_sites_fil
     str
         path to the new FASTA file from which sites have been excluded
     """
-    # Load zero-based excluded sites.
+    # Read 1-based excluded sites and store as 0-based sites.
     excluded_sites = load_mask_sites(excluded_sites_file)
 
     # Return the original alignment file, if no excluded sites were found.
@@ -394,7 +399,8 @@ def mask_sites_in_multiple_sequence_alignment(alignment_file, excluded_sites_fil
     return masked_alignment_file
 
 
-def register_arguments(parser):
+def register_parser(parent_subparsers):
+    parser = parent_subparsers.add_parser("tree", help=__doc__)
     parser.add_argument('--alignment', '-a', required=True, help="alignment in fasta or VCF format")
     parser.add_argument('--method', default='iqtree', choices=["fasttree", "raxml", "iqtree"], help="tree builder to use")
     parser.add_argument('--output', '-o', type=str, help='file name to write tree to')
@@ -415,6 +421,7 @@ def register_arguments(parser):
     parser.epilog = """For example, to build a tree with IQ-TREE, use the following format:
     augur tree --method iqtree --alignment <alignment> --substitution-model <model> --output <tree> --tree-builder-args="<extra arguments>"
     """
+    return parser
 
 def run(args):
     # check alignment type, set flags, read in if VCF
