@@ -1,16 +1,13 @@
 import argparse
-import re
 import Bio
 import Bio.Phylo
-from datetime import datetime
 import gzip
 import os, json, sys
 import pandas as pd
 import subprocess
 import shlex
 from contextlib import contextmanager
-from treetime.utils import numeric_date
-from collections import defaultdict
+from collections import defaultdict, OrderedDict
 from pkg_resources import resource_stream
 from io import TextIOWrapper
 from .__version__ import __version__
@@ -18,52 +15,9 @@ from .__version__ import __version__
 from augur.io import open_file
 
 from augur.util_support.color_parser import ColorParser
-from augur.util_support.date_disambiguator import DateDisambiguator
-from augur.util_support.metadata_file import MetadataFile
 from augur.util_support.node_data_reader import NodeDataReader
-from augur.util_support.shell_command_runner import ShellCommandRunner
 
 
-class AugurException(Exception):
-    pass
-
-
-def is_vcf(filename):
-    """Convenience method to check if a file is a vcf file.
-
-    >>> is_vcf(None)
-    False
-    >>> is_vcf("./foo")
-    False
-    >>> is_vcf("./foo.vcf")
-    True
-    >>> is_vcf("./foo.vcf.GZ")
-    True
-    """
-    return bool(filename) and any(filename.lower().endswith(x) for x in ('.vcf', '.vcf.gz'))
-
-def read_vcf(filename):
-    if filename.lower().endswith(".gz"):
-        import gzip
-        file = gzip.open(filename, mode="rt", encoding='utf-8')
-    else:
-        file = open(filename, encoding='utf-8')
-
-    chrom_line = next(line for line in file if line.startswith("#C"))
-    file.close()
-    headers = chrom_line.strip().split("\t")
-    sequences = headers[headers.index("FORMAT") + 1:]
-
-    # because we need 'seqs to remove' for VCF
-    return sequences, sequences.copy()
-
-def myopen(fname, mode):
-    if fname.endswith('.gz'):
-        import gzip
-        return gzip.open(fname, mode, encoding='utf-8')
-    else:
-        return open(fname, mode, encoding='utf-8')
-
 def get_json_name(args, default=None):
     if args.output_node_data:
         return args.output_node_data
@@ -75,94 +29,6 @@ def get_json_name(args, default=None):
             raise ValueError("Please specify a name for the JSON file containing the results.")
 
 
-def ambiguous_date_to_date_range(uncertain_date, fmt, min_max_year=None):
-    return DateDisambiguator(uncertain_date, fmt=fmt, min_max_year=min_max_year).range()
-
-def read_metadata(fname, query=None, as_data_frame=False):
-    return MetadataFile(fname, query, as_data_frame).read()
-
-def is_date_ambiguous(date, ambiguous_by="any"):
-    """
-    Returns whether a given date string in the format of YYYY-MM-DD is ambiguous by a given part of the date (e.g., day, month, year, or any parts).
-
-    Parameters
-    ----------
-    date : str
-        Date string in the format of YYYY-MM-DD
-    ambiguous_by : str
-        Field of the date string to test for ambiguity ("day", "month", "year", "any")
-    """
-    date_components = date.split('-', 2)
-
-    if len(date_components) == 3:
-        year, month, day = date_components
-    elif len(date_components) == 2:
-        year, month = date_components
-        day = "XX"
-    else:
-        year = date_components[0]
-        month = "XX"
-        day = "XX"
-
-    # Determine ambiguity hierarchically such that, for example, an ambiguous
-    # month implicates an ambiguous day even when day information is available.
-    return any((
-        "X" in year,
-        "X" in month and ambiguous_by in ("any", "month", "day"),
-        "X" in day and ambiguous_by in ("any", "day")
-    ))
-
-def get_numerical_date_from_value(value, fmt=None, min_max_year=None):
-    value = str(value)
-    if re.match(r'^-*\d+\.\d+$', value):
-        # numeric date which can be negative
-        return float(value)
-    if value.isnumeric():
-        # year-only date is ambiguous
-        value = fmt.replace('%Y', value).replace('%m', 'XX').replace('%d', 'XX')
-    if 'XX' in value:
-        ambig_date = ambiguous_date_to_date_range(value, fmt, min_max_year)
-        if ambig_date is None or None in ambig_date:
-            return [None, None] #don't send to numeric_date or will be set to today
-        return [numeric_date(d) for d in ambig_date]
-    try:
-        return numeric_date(datetime.strptime(value, fmt))
-    except:
-        return None
-
-def get_numerical_dates(meta_dict, name_col = None, date_col='date', fmt=None, min_max_year=None):
-    if fmt:
-        numerical_dates = {}
-
-        if isinstance(meta_dict, dict):
-            for k,m in meta_dict.items():
-                v = m[date_col]
-                numerical_dates[k] = get_numerical_date_from_value(
-                    v,
-                    fmt,
-                    min_max_year
-                )
-        elif isinstance(meta_dict, pd.DataFrame):
-            strains = meta_dict.index.values
-            dates = meta_dict[date_col].apply(
-                lambda date: get_numerical_date_from_value(
-                    date,
-                    fmt,
-                    min_max_year
-                )
-            ).values
-            numerical_dates = dict(zip(strains, dates))
-    else:
-        if isinstance(meta_dict, dict):
-            numerical_dates = {k:float(v) for k,v in meta_dict.items()}
-        elif isinstance(meta_dict, pd.DataFrame):
-            strains = meta_dict.index.values
-            dates = meta_dict[date_col].astype(float)
-            numerical_dates = dict(zip(strains, dates))
-
-    return numerical_dates
-
-
 class InvalidTreeError(Exception):
     """Represents an error loading a phylogenetic tree from a filename.
     """
@@ -220,8 +86,8 @@ def read_tree(fname, min_terminals=3):
     return T
 
 
-def read_node_data(fnames, tree=None):
-    return NodeDataReader(fnames, tree).read()
+def read_node_data(fnames, tree=None, skip_validation=False):
+    return NodeDataReader(fnames, tree, skip_validation).read()
 
 
 def write_json(data, file_name, indent=(None if os.environ.get("AUGUR_MINIFY_JSON") else 2), include_version=True):
@@ -256,9 +122,9 @@ def write_json(data, file_name, indent=(None if os.environ.get("AUGUR_MINIFY_JSO
 
     if include_version:
         data["generated_by"] = {"program": "augur", "version": get_augur_version()}
-
     with open(file_name, 'w', encoding='utf-8') as handle:
-        json.dump(data, handle, indent=indent, sort_keys=True)
+        sort_keys = False if isinstance(data, OrderedDict) else True
+        json.dump(data, handle, indent=indent, sort_keys=sort_keys)
 
 
 def load_features(reference, feature_names=None):
@@ -276,14 +142,20 @@ def load_features(reference, feature_names=None):
         except ImportError:
             print("ERROR: Package BCBio.GFF not found! Please install using \'pip install bcbio-gff\' before re-running.")
             return None
-        limit_info = dict( gff_type = ['gene'] )
+        limit_info = dict( gff_type = ['gene', 'source'] )
 
         with open(reference, encoding='utf-8') as in_handle:
             for rec in GFF.parse(in_handle, limit_info=limit_info):
                 for feat in rec.features:
-                    if feature_names is not None: #check both tags; user may have used either
+                    # Check for gene names stored in qualifiers commonly used by
+                    # virus-specific gene maps first (e.g., 'gene',
+                    # 'gene_name'). Then, check for qualifiers used by non-viral
+                    # pathogens (e.g., 'locus_tag').
+                    if feature_names is not None:
                         if "gene" in feat.qualifiers and feat.qualifiers["gene"][0] in feature_names:
                             fname = feat.qualifiers["gene"][0]
+                        elif "gene_name" in feat.qualifiers and feat.qualifiers["gene_name"][0] in feature_names:
+                            fname = feat.qualifiers["gene_name"][0]
                         elif "locus_tag" in feat.qualifiers and feat.qualifiers["locus_tag"][0] in feature_names:
                             fname = feat.qualifiers["locus_tag"][0]
                         else:
@@ -291,8 +163,13 @@ def load_features(reference, feature_names=None):
                     else:
                         if "gene" in feat.qualifiers:
                             fname = feat.qualifiers["gene"][0]
+                        elif "gene_name" in feat.qualifiers:
+                            fname = feat.qualifiers["gene_name"][0]
                         else:
                             fname = feat.qualifiers["locus_tag"][0]
+                    if feat.type == "source":
+                        fname = "nuc"
+
                     if fname:
                         features[fname] = feat
 
@@ -370,108 +247,6 @@ def read_lat_longs(overrides=None, use_defaults=True):
 def read_colors(overrides=None, use_defaults=True):
     return ColorParser(mapping_filename=overrides, use_defaults=use_defaults).mapping
 
-def write_VCF_translation(prot_dict, vcf_file_name, ref_file_name):
-    """
-    Writes out a VCF-style file (which seems to be minimally handleable
-    by vcftools and pyvcf) of the AA differences between sequences and the reference.
-    This is a similar format created/used by read_in_vcf except that there is one
-    of these dicts (with sequences, reference, positions) for EACH gene.
-
-    Also writes out a fasta of the reference alignment.
-
-    EBH 12 Dec 2017
-    """
-    import numpy as np
-
-    #for the header
-    seqNames = list(prot_dict[list(prot_dict.keys())[0]]['sequences'].keys())
-
-    #prepare the header of the VCF & write out
-    header=["#CHROM","POS","ID","REF","ALT","QUAL","FILTER","INFO","FORMAT"]+seqNames
-    with open(vcf_file_name, 'w', encoding='utf-8') as the_file:
-        the_file.write( "##fileformat=VCFv4.2\n"+
-                        "##source=NextStrain_Protein_Translation\n"+
-                        "##FORMAT=<ID=GT,Number=1,Type=String,Description=\"Genotype\">\n")
-        the_file.write("\t".join(header)+"\n")
-
-    refWrite = []
-    vcfWrite = []
-
-    #go through for every gene/protein
-    for fname, prot in prot_dict.items():
-        sequences = prot['sequences']
-        ref = prot['reference']
-        positions = prot['positions']
-
-        #write out the reference fasta
-        refWrite.append(">"+fname)
-        refWrite.append(ref)
-
-        #go through every variable position
-        #There are no deletions here, so it's simpler than for VCF nuc sequenes!
-        for pi in positions:
-            pos = pi+1 #change numbering to match VCF not python
-            refb = ref[pi] #reference base at this position
-
-            #try/except is (much) faster than list comprehension!
-            pattern = []
-            for k,v in sequences.items():
-                try:
-                    pattern.append(sequences[k][pi])
-                except KeyError:
-                    pattern.append('.')
-            pattern = np.array(pattern)
-
-            #get the list of ALTs - minus any '.'!
-            uniques = np.unique(pattern)
-            uniques = uniques[np.where(uniques!='.')]
-
-            #Convert bases to the number that matches the ALT
-            j=1
-            for u in uniques:
-                pattern[np.where(pattern==u)[0]] = str(j)
-                j+=1
-            #Now convert these calls to #/# (VCF format)
-            calls = [ j+"/"+j if j!='.' else '.' for j in pattern ]
-            if len(uniques)==0:
-                print("UNEXPECTED ERROR WHILE CONVERTING TO VCF AT POSITION {}".format(str(pi)))
-                break
-
-            #put it all together and write it out
-            output = [fname, str(pos), ".", refb, ",".join(uniques), ".", "PASS", ".", "GT"] + calls
-
-            vcfWrite.append("\t".join(output))
-
-    #write it all out
-    with open(ref_file_name, 'w', encoding='utf-8') as the_file:
-        the_file.write("\n".join(refWrite))
-
-    with open(vcf_file_name, 'a', encoding='utf-8') as the_file:
-        the_file.write("\n".join(vcfWrite))
-
-    if vcf_file_name.lower().endswith('.gz'):
-        import os
-        #must temporarily remove .gz ending, or gzip won't zip it!
-        os.rename(vcf_file_name, vcf_file_name[:-3])
-        call = ["gzip", vcf_file_name[:-3]]
-        run_shell_command(" ".join(call), raise_errors = True)
-
-shquote = shlex.quote
-
-def run_shell_command(cmd, raise_errors=False, extra_env=None):
-    """
-    Run the given command string via Bash with error checking.
-
-    Returns True if the command exits normally.  Returns False if the command
-    exits with failure and "raise_errors" is False (the default).  When
-    "raise_errors" is True, exceptions are rethrown.
-
-    If an *extra_env* mapping is passed, the provided keys and values are
-    overlayed onto the default subprocess environment.
-    """
-    return ShellCommandRunner(cmd, raise_errors=raise_errors, extra_env=extra_env).run()
-
-
 def first_line(text):
     """
     Returns the first line of the given text, ignoring leading and trailing
@@ -551,7 +326,7 @@ def annotate_parents_for_tree(tree):
     return tree
 
 
-def json_to_tree(json_dict, root=True):
+def json_to_tree(json_dict, root=True, parent_cumulative_branch_length=None):
     """Returns a Bio.Phylo tree corresponding to the given JSON dictionary exported
     by `tree_to_json`.
 
@@ -589,6 +364,17 @@ def json_to_tree(json_dict, root=True):
     True
     >>> tree.clades[0].branch_length > 0
     True
+
+    Branch lengths should be the length of the branch to each node and not the
+    length from the root. The cumulative branch length from the root gets its
+    own attribute.
+
+    >>> tip = [tip for tip in tree.find_clades(terminal=True) if tip.name == "USA/2016/FLWB042"][0]
+    >>> round(tip.cumulative_branch_length, 6)
+    0.004747
+    >>> round(tip.branch_length, 6)
+    0.000186
+
     """
     # Check for v2 JSON which has combined metadata and tree data.
     if root and "meta" in json_dict and "tree" in json_dict:
@@ -602,10 +388,6 @@ def json_to_tree(json_dict, root=True):
     else:
         node.name = json_dict["strain"]
 
-    if "children" in json_dict:
-        # Recursively add children to the current node.
-        node.clades = [json_to_tree(child, root=False) for child in json_dict["children"]]
-
     # Assign all non-children attributes.
     for attr, value in json_dict.items():
         if attr != "children":
@@ -614,12 +396,27 @@ def json_to_tree(json_dict, root=True):
     # Only v1 JSONs support a single `attr` attribute.
     if hasattr(node, "attr"):
         node.numdate = node.attr.get("num_date")
-        node.branch_length = node.attr.get("div")
+        node.cumulative_branch_length = node.attr.get("div")
 
         if "translations" in node.attr:
             node.translations = node.attr["translations"]
     elif hasattr(node, "node_attrs"):
-        node.branch_length = node.node_attrs.get("div")
+        node.cumulative_branch_length = node.node_attrs.get("div")
+
+    node.branch_length = 0.0
+    if parent_cumulative_branch_length is not None and hasattr(node, "cumulative_branch_length"):
+        node.branch_length = node.cumulative_branch_length - parent_cumulative_branch_length
+
+    if "children" in json_dict:
+        # Recursively add children to the current node.
+        node.clades = [
+            json_to_tree(
+                child,
+                root=False,
+                parent_cumulative_branch_length=node.cumulative_branch_length
+            )
+            for child in json_dict["children"]
+        ]
 
     if root:
         node = annotate_parents_for_tree(node)
@@ -646,8 +443,8 @@ def read_bed_file(bed_file):
     bed_file : str
         Path to the BED file
 
-    Returns:
-    --------
+    Returns
+    -------
     list[int]:
         Sorted list of unique zero-indexed sites
     """
@@ -677,8 +474,8 @@ def read_mask_file(mask_file):
     mask_file : str
         Path to the masking file
 
-    Returns:
-    --------
+    Returns
+    -------
     list[int]:
         Sorted list of unique zero-indexed sites
     """
@@ -726,7 +523,7 @@ def read_strains(*files, comment_char="#"):
     set of distinct strains.
 
     Strain names can be commented with full-line or inline comments. For
-    example, the following is a valid strain names file:
+    example, the following is a valid strain names file::
 
         # this is a comment at the top of the file
         strain1  # exclude strain1 because it isn't sequenced properly
